#![no_main]
#![no_std]

pub mod config;
pub mod multicast;

use log::info;
use uefi::fs::PathBuf;
use uefi::prelude::*;
use uefi::table::boot::LoadImageSource;
use uefi::{Error, Result, Status};

fn load_efi_app_from_path(boot_services: &BootServices, app_path: PathBuf) -> Result {
    let mut fs_handler = boot_services.get_image_file_system(boot_services.image_handle())?;
    let app_content = match fs_handler.read(app_path) {
        Err(_e) => return Err(Error::from(Status::PROTOCOL_ERROR)),
        Ok(x) => x,
    };
    let image_source = LoadImageSource::FromBuffer {
        buffer: &app_content,
        file_path: None,
    };
    let img_handler = boot_services.load_image(boot_services.image_handle(), image_source)?;
    boot_services.start_image(img_handler)
}

#[entry]
fn main(_handle: Handle, mut system_table: SystemTable<Boot>) -> Status {
    info!("init service");

    uefi_services::init(&mut system_table).unwrap();

    info!("init boot services");
    let boot_services = system_table.boot_services();

    info!("init runtime services");
    let runtime_services = system_table.runtime_services();

    let conf = config::read_config(runtime_services).unwrap();
    let app_path = multicast::send_mcast(boot_services, &conf).unwrap();
    load_efi_app_from_path(boot_services, app_path).unwrap();

    Status::SUCCESS
}
